import numpy as np
import matplotlib.pyplot as plt
import datetime as dt
import struct
import scipy.fftpack as ft
import scipy.signal as sig
import obspy
from obspy.signal.tf_misfit import cwt
import sys
import fileSet as fs
import regularFileSet as rfs
import os

ChEastUTMNorthUTM = np.load('inter_results/PassiveChEastUTMNorthUTM.npz')
Channels = ChEastUTMNorthUTM['arr_0']
EastUTM = ChEastUTMNorthUTM['arr_1']
NorthUTM = ChEastUTMNorthUTM['arr_2'] 


nTxtFileHeader = 3200
nBinFileHeader = 400
nTraceHeader = 240
def readTrace(infile,nSamples,dataLen,traceNumber,endian,startSample):
    '''infile is .sgy, nSamples is the number of samples per sensor, and traceNumber is the sensor number (start with 1),dataLen is number of bytes per data sample'''

    fin = open(infile, 'rb') # open file for reading binary mode
    startData = nTxtFileHeader+nBinFileHeader+nTraceHeader+(traceNumber-1)*(nTraceHeader+dataLen*nSamples)+startSample*dataLen
    fin.seek(startData)
    thisTrace = np.zeros(nSamples)
    for i in range(nSamples):
       	# was >f before
       	thisTrace[i] = struct.unpack(endian+'f',fin.read(dataLen))[0]
    fin.close()
    return thisTrace

# channels fo interest
nChannels = 620
channelSets = [(100,110),(230,240)] # (start channel, stop channel) to stack over for different locations

# create list of files
datapath = '/data/biondo/DAS/'
parts = [datapath,'year4','/','month','/','day','/','cbt_processed_','year4','month','day','_','hour24start0','minute','second','.','millisecond','+0000.sgy']
secondsPerFile = 300
minutesStacked = 10
filesPerInterval = minutesStacked*60/secondsPerFile
nDays = 7
secondsPerDay = 24*60*60
nIntervals = nDays*24*60/minutesStacked
nFiles = nDays*secondsPerDay/secondsPerFile
myFileSet = rfs.regularFileSet(parts,2016,11,27,7,59,16,572,secondsPerFile,nFiles)
startTime = dt.datetime(2016,11,27,7,59,16,572*1000)
startReadTime = startTime
endTime = dt.datetime(2016,12,4,7,59,16,552*1000)
fileList = myFileSet.getFileNamesInRange(startTime,endTime)
#print(fileList)

# get header info from files
nBytesPerFile = os.path.getsize(fileList[0])
bytesPerChannel = (nBytesPerFile-nTxtFileHeader-nBinFileHeader-240*nChannels)/nChannels
samplesPerChannel = bytesPerChannel/4
samplesPerSecond = samplesPerChannel/secondsPerFile 
print("samples per second "+str(samplesPerSecond))
NyquistFrq = float(samplesPerSecond)/2.0
samplesPerFile = samplesPerSecond*secondsPerFile

# for each interval calculate spectra in channel segments of interest
specOverall = np.zeros((len(channelSets),nIntervals,samplesPerFile*filesPerInterval/2)) # strain spec
specDtOverall = np.zeros((len(channelSets),nIntervals,(samplesPerFile*filesPerInterval-1)/2)) # strain rate spec
for interval in range(nIntervals):
    print(interval)
    intervalFileList = fileList[interval*filesPerInterval:(interval+1)*filesPerInterval]

    for cs,channelSet in enumerate(channelSets):
        startChannel = channelSet[0]
        endChannel = channelSet[1]
        myChannels = np.arange(startChannel,endChannel+1)
        nChannels = myChannels.size
        data = np.zeros((nChannels,samplesPerFile*filesPerInterval))


        #seconds = np.linspace(0,samplesPerFile*len(fileList)/float(samplesPerSecond),samplesPerFile*len(fileList))
        for i,f in enumerate(intervalFileList):
            startReadingSample = 0
            for c in range(startChannel,endChannel+1):
                data[c-startChannel,i*samplesPerFile:(i+1)*samplesPerFile] = readTrace(f,samplesPerFile,4,c,'>',startReadingSample)

        
        # get rid of average
        dataAvg = np.sum(data,axis=1)
        dataAvg = dataAvg/float(samplesPerFile*len(intervalFileList))
        data = data - dataAvg.reshape((nChannels,1))


        dataDt = (data[:,1:] - data[:,:-1])/float(samplesPerSecond) # create data that's strain rate        
        # get the spectrum for strain and strain rate   
        thisSpec = np.absolute(ft.fft(data))
        specOverall[cs,interval,:] = np.sum(thisSpec[:,:samplesPerFile*filesPerInterval/2],axis=0)/nChannels
        thisSpec = np.absolute(ft.fft(dataDt))
        specDtOverall[cs,interval,:] = np.sum(thisSpec[:,:(samplesPerFile*filesPerInterval-1)/2],axis=0)/nChannels

        

np.save('inter_results/week_spec_strain.npy',specOverall)
np.save('inter_results/week_spec_strain_rate.npy',specDtOverall)
        
