import numpy as np
import matplotlib.pyplot as plt
import struct
import sys
import scipy.fftpack as ft
import datetime as dt
import os

# geometry info also in here, but for now just get which channels are outside of Green building
ActiveChEastUTMNorthUTM = np.load('inter_results/ActiveChEastUTMNorthUTM.npz')
Channels = ActiveChEastUTMNorthUTM['arr_0']


nTxtFileHeader = 3200
nBinFileHeader = 400
nTraceHeader = 240
def readTrace(infile,nSamples,dataLen,traceNumber,endian):
    '''infile is .sgy, nSamples is the number of samples per sensor, and traceNumber is the sensor number (start with 1),dataLen is number of bytes per data sample'''

    fin = open(infile, 'rb') # open file for reading binary mode
    startData = nTxtFileHeader+nBinFileHeader+nTraceHeader+(traceNumber-1)*(nTraceHeader+dataLen*nSamples)
    fin.seek(startData)
    thisTrace = np.zeros(nSamples)
    for i in range(nSamples):
       	# was >f before
       	thisTrace[i] = struct.unpack(endian+'f',fin.read(dataLen))[0]
    fin.close()
    return thisTrace

# basic info about which data to read
filename = '/data/biondo/DAS/CALIBRATION-SEGY-5/cbt_processed_53.sgy'
secondsPerFile = 60
samplesPerSec = 2500
NyquistFrq = float(samplesPerSec)*0.5
samplesPerFile = samplesPerSec*secondsPerFile
startChannel = Channels[0]
endChannel = Channels[-1]
nChannels = endChannel-startChannel+1

# read a little header info about when the file starts
fin = open(filename,'rb')
fin.seek(nTxtFileHeader+nBinFileHeader+161)
hr = struct.unpack('H',fin.read(2))[0]
minute = struct.unpack('H',fin.read(2))[0]
sec = struct.unpack('H',fin.read(2))[0]
fin.close()
fileStartTime = dt.datetime(2016,9,1,hr,minute,sec) # but... overwrite month, day, year (should figure out what to do w/ leapday eventually)

# rest of info about exactly which times to read and plot
startTime = dt.datetime(2016,9,1,20,54,03)
startIdx = int((startTime-fileStartTime).total_seconds()*samplesPerSec)
endTime = dt.datetime(2016,9,1,20,54,13)
endIdx = int((endTime-fileStartTime).total_seconds()*samplesPerSec)

# actaully read data
data = np.zeros((nChannels,samplesPerFile))
for c in range(startChannel,endChannel+1):
    data[c-startChannel,:] = readTrace(filename,samplesPerFile,4,c,'>')

# get rid of average for each channel
dataAvg = np.sum(data,axis=1)
dataAvg = dataAvg/float(samplesPerFile)
data = data - dataAvg.reshape((nChannels,1))
dataStdDev = np.sqrt(np.sum(data*data,axis=1))/float(samplesPerFile)
data = data/dataStdDev.reshape((nChannels,1))


# bandpass filter
spec = ft.fft(data)
lowFrq = 0.2 # in Hz
highFrq = 5.0 # in Hz
SamplesPerHz = 0.5*samplesPerFile/NyquistFrq
spec[:,:int(lowFrq*SamplesPerHz)] = 0
spec[:,-int(lowFrq*SamplesPerHz):] = 0
spec[:,int(highFrq*SamplesPerHz):-int(highFrq*SamplesPerHz)] = 0
data = np.real(ft.ifft(spec))


# plot data in a line
plt.imshow(data[:,startIdx:endIdx],aspect='auto',interpolation='nearest',cmap=plt.get_cmap('seismic'),extent=[0,(endTime-startTime).total_seconds(),endChannel,startChannel],vmin=-1000,vmax=1000)
plt.ylabel('channel (1 m/channel)')
plt.xlabel('time (seconds)')
plt.colorbar()
plt.title('data from '+str(startTime)+' to '+str(endTime))
plt.savefig('fig/activeEx.pdf')
plt.clf()

