import numpy as np
import matplotlib.pyplot as plt
import struct
import datetime as dt
import scipy.fftpack as ft

nTxtFileHeader = 3200
nBinFileHeader = 400
nTraceHeader = 240
def readTrace(infile,nSamples,dataLen,traceNumber,endian,startSample):
    '''infile is .sgy, nSamples is the number of samples per sensor, and traceNumber is the sensor number (start with 1),dataLen is number of bytes per data sample'''

    fin = open(infile, 'rb') # open file for reading binary mode
    startData = nTxtFileHeader+nBinFileHeader+nTraceHeader+(traceNumber-1)*(nTraceHeader+dataLen*nSamples)+startSample*dataLen
    fin.seek(startData)
    thisTrace = np.zeros(nSamples)
    for i in range(nSamples):
       	thisTrace[i] = struct.unpack(endian+'f',fin.read(dataLen))[0]
    fin.close()
    return thisTrace


secondsPerFile = 60 # need to automate
fileList = ['/data/biondo/DAS/2016/10/04/weirdHeaders/cbt_processed_20161004_125643.908+0000.sgy','/data/biondo/DAS/2016/10/04/weirdHeaders/cbt_processed_20161004_125743.908+0000.sgy']
startReadTime = dt.datetime(2016,10,4,12,56,43,908*1000)


import os
nBytesPerFile = os.path.getsize(fileList[0])
nChannels = 2480
bytesPerChannel = (nBytesPerFile-nTxtFileHeader-nBinFileHeader-240*nChannels)/nChannels
samplesPerChannel = bytesPerChannel/4
samplesPerSecond = samplesPerChannel/secondsPerFile 
print("samples per second "+str(samplesPerSecond))
NyquistFrq = float(samplesPerSecond)/2.0
samplesPerFile = samplesPerSecond*secondsPerFile
startChannel = 2240
endChannel = 2370  # first channel not included
nChannels = endChannel-startChannel
data = np.zeros((nChannels,samplesPerFile*len(fileList)))
startTime = dt.datetime(2016,10,4,12,57,42,200*1000)
startSample = int(samplesPerSecond*(startTime-startReadTime).total_seconds())

endTime = dt.datetime(2016,10,4,12,57,43,0)
endSample = int(samplesPerSecond*(endTime-startReadTime).total_seconds())
for i,f in enumerate(fileList):
    startReadingSample = 0
    for c in range(startChannel,endChannel):
        data[c-startChannel,i*samplesPerFile:(i+1)*samplesPerFile] = readTrace(f,samplesPerFile,4,c,'>',startReadingSample)
# get rid of average of data for each channel (like DC drift)
dataAvg = np.sum(data,axis=1)
dataAvg = dataAvg/float(samplesPerFile*len(fileList))
data = data - dataAvg.reshape((endChannel-startChannel,1))
dataStdDev = np.sqrt(np.sum(data*data,axis=1))/float(samplesPerFile*len(fileList))
data = data/dataStdDev.reshape((endChannel-startChannel,1))


# filter/bandpass data 
spec = ft.fft(data)
lowFrq = 0.5 # in Hz
highFrq = 200 # in Hz
SamplesPerHz = 0.5*samplesPerFile*len(fileList)/NyquistFrq
spec[:,:int(lowFrq*SamplesPerHz)] = 0
spec[:,-int(lowFrq*SamplesPerHz):] = 0
spec[:,int(highFrq*SamplesPerHz):-int(highFrq*SamplesPerHz)] = 0
data = np.real(ft.ifft(spec))


# plot the data in the channel range of interest
plt.imshow(data[:,startSample:endSample],aspect='auto',interpolation='nearest',cmap=plt.get_cmap('seismic'),extent=[0,(endTime-startTime).total_seconds(),endChannel,startChannel],vmin=-1000,vmax=1000)
plt.xlabel('seconds after '+str(startTime))
plt.ylabel('channel (1 m/channel)')
plt.colorbar()
plt.title('Data from Betsy Shot South of Mitchell')
plt.savefig('fig/BetsySouthOfMitchell.pdf')
plt.clf()
