import numpy as np
import matplotlib.pyplot as plt
import datetime as dt

strainSpec = np.load('inter_results/week_spec_strain.npy')
strainRateSpec = np.load('inter_results/week_spec_strain_rate.npy')

(nChannelSets,nIntervals,nFrqs) = strainSpec.shape
samplesPerSec = 50.0
maxFrq = samplesPerSec/2.0
HzPerBin = maxFrq/nFrqs

channelSets = [(100,110),(230,240)]
minutesStacked = 10
HrsPerInterval = float(minutesStacked)/60.0
IntervalsPerDay = 24*60/minutesStacked
startTime = dt.datetime(2016,11,27,7,59,16,572*1000)


for cs,channelSet in enumerate(channelSets):
    perc = 2.5
    maxVal = np.log(np.percentile(strainSpec[cs,:,:],100-perc))
    minVal = np.log(np.percentile(strainSpec[cs,:,:],perc))
    plt.imshow(np.transpose(np.log(strainSpec[cs,:,:])),aspect='auto',interpolation='nearest',vmin=minVal,vmax=maxVal)
    plt.colorbar()
    plt.title('stack of strain spec. channels '+str(channelSet[0])+' to '+str(channelSet[1]))
    plt.xlabel('hours after '+str(startTime)+' UTC')
    plt.ylabel('frequency (Hz)')
    plt.yticks(np.arange(0,nFrqs,3000),HzPerBin*np.arange(0.0,nFrqs,3000))
    plt.xticks(np.arange(0,nIntervals,IntervalsPerDay),HrsPerInterval*np.arange(0.0,nIntervals,IntervalsPerDay))
    plt.savefig('fig/strainRateSpec'+str(channelSet[0])+'To'+str(channelSet[1])+'.pdf')
    plt.clf()

    maxVal = np.log(np.percentile(strainRateSpec[cs,:,:],100-perc))
    minVal = np.log(np.percentile(strainRateSpec[cs,:,:],perc))
    plt.imshow(np.transpose(np.log(strainRateSpec[cs,:,:])),aspect='auto',interpolation='nearest',vmax=maxVal,vmin=minVal)
    plt.colorbar()
    plt.title('stack of strain rate spec. channels '+str(channelSet[0])+' to '+str(channelSet[1]))
    plt.xlabel('hours after '+str(startTime)+' UTC')
    plt.ylabel('frequency (Hz)')
    plt.yticks(np.arange(0,nFrqs,3000),HzPerBin*np.arange(0.0,nFrqs,3000))
    plt.xticks(np.arange(0,nIntervals,IntervalsPerDay),HrsPerInterval*np.arange(0.0,nIntervals,IntervalsPerDay))
    plt.savefig('fig/strainRateSpec'+str(channelSet[0])+'To'+str(channelSet[1])+'.pdf')
    plt.clf()
