import numpy as np
import matplotlib.pyplot as plt
import struct
import sys
import datetime as dt
import scipy.fftpack as ft
from scipy.signal import butter, lfilter

nTxtFileHeader = 3200
nBinFileHeader = 400
nTraceHeader = 240
def readTrace(infile,nSamples,dataLen,traceNumber,endian):
    '''infile is .sgy, nSamples is the number of samples per sensor, and traceNumber is the sensor number (start with 1),dataLen is number of bytes per data sample'''
    fin = open(infile, 'rb') # open file for reading binary mode
    startData = nTxtFileHeader+nBinFileHeader+nTraceHeader+(traceNumber-1)*(nTraceHeader+dataLen*nSamples)
    fin.seek(startData)
    thisTrace = np.zeros(nSamples)
    for i in range(nSamples):
       	thisTrace[i] = struct.unpack(endian+'f',fin.read(dataLen))[0]
    fin.close()
    return thisTrace


# specify basic info about what data to grab
secondsPerFile = 60 # need to automate?
startTime = dt.datetime(2016,10,4,13,52,00,908*1000)
endTime = dt.datetime(2016,10,4,13,58,00,908*1000)
fileList = ['/data/biondo/DAS/2016/10/06/weirdHeaders/cbt_processed_20161006_160450.113+0000.sgy']
nChannels = 2480
startChannel = 100
endChannnel = 225
import os
nBytesPerFile = os.path.getsize(fileList[0])
bytesPerChannel = (nBytesPerFile-nTxtFileHeader-nBinFileHeader-240*nChannels)/nChannels
samplesPerChannel = bytesPerChannel/4
samplesPerSecond = samplesPerChannel/secondsPerFile 
NyquistFrq = float(samplesPerSecond)/2.0
samplesPerFile = samplesPerSecond*secondsPerFile

# read in data
nChannels = endChannel-startChannel
data = np.zeros((nChannels,samplesPerFile*len(fileList)))
for i,f in enumerate(fileList):
    for c in range(startChannel,endChannel):
        data[c-startChannel,i*samplesPerFile:(i+1)*samplesPerFile] = readTrace(f,samplesPerFile,4,c,'>')

# get rid of average of each channel
dataAvg = np.sum(data,axis=1)
dataAvg = dataAvg/float(samplesPerFile*len(fileList))
data = data - dataAvg.reshape((endChannel-startChannel,1))
dataStdDev = np.sqrt(np.sum(data*data,axis=1))/float(samplesPerFile*len(fileList))
data = data/dataStdDev.reshape((endChannel-startChannel,1))

# filter data 
spec = ft.fft(data)
lowFrq = 0.5 # in Hz
highFrq = 200 # in Hz
SamplesPerHz = 0.5*samplesPerFile*len(fileList)/NyquistFrq
spec[:,:int(lowFrq*SamplesPerHz)] = 0
spec[:,-int(lowFrq*SamplesPerHz):] = 0
spec[:,int(highFrq*SamplesPerHz):-int(highFrq*SamplesPerHz)] = 0
data = np.real(ft.ifft(spec))



# plot energy in band of interest
windowSec = 1 # for each second of data
HzPerSample = NyquistFrq/(samplesPerSecond*windowSec/2)
nWindows = int(secondsPerFile*len(fileList)/windowSec)
samplesPerWindow = windowSec*samplesPerSecond
windowSpec = np.zeros((nChannels,nWindows))
for i in range(nWindows):
    windowSpecs = ft.fft(data[:,i*samplesPerWindow:(i+1)*samplesPerWindow])
    windowSpec[:,i] = np.sum(np.abs(windowSpecs[:,int(lowFrq/HzPerSample):int(highFrq/HzPerSample)]),axis=1)

plt.imshow(windowSpec,aspect='auto',interpolation='nearest',vmin=5000000,vmax=20000000,cmap=plt.get_cmap('plasma'),extent=[0,secondsPerFile,endChannel-1,startChannel])
plt.colorbar()
plt.xlabel('seconds')
plt.ylabel('channel (1 m/channel)')
plt.title('power at different times')
plt.savefig('fig/tapPower.pdf')

